Package: methylGSA
Type: Package
Title: Gene Set Analysis Using the Outcome of Differential Methylation
Version: 1.31.0
Authors@R: c(
    person("Xu", "Ren", 
        email = "xuren2120@gmail.com", role = c("aut", "cre")),
    person("Pei Fen", "Kuan", 
        email = "peifen.kuan@stonybrook.edu", role = "aut"))
Description: The main functions for methylGSA are methylglm and methylRRA.
    methylGSA implements logistic regression adjusting number of probes as 
    a covariate.
    methylRRA adjusts multiple p-values of each gene by Robust Rank Aggregation.
    For more detailed help information, please see the vignette.
Encoding: UTF-8
Imports: RobustRankAggreg, ggplot2, stringr, stats, clusterProfiler,
        missMethyl, org.Hs.eg.db, reactome.db, BiocParallel, GO.db,
        AnnotationDbi, shiny,
        IlluminaHumanMethylation450kanno.ilmn12.hg19,
        IlluminaHumanMethylationEPICanno.ilm10b4.hg19
Depends: R (>= 3.5)
Suggests: knitr, rmarkdown, testthat, enrichplot
License: GPL-2
URL: https://github.com/reese3928/methylGSA
BugReports: https://github.com/reese3928/methylGSA/issues
RoxygenNote: 6.1.0
VignetteBuilder: knitr
biocViews:
        DNAMethylation,DifferentialMethylation,GeneSetEnrichment,Regression,
        GeneRegulation,Pathways
git_url: https://git.bioconductor.org/packages/methylGSA
git_branch: devel
git_last_commit: 7792ec0
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-11
NeedsCompilation: no
Packaged: 2026-05-11 22:45:46 UTC; biocbuild
Author: Xu Ren [aut, cre],
  Pei Fen Kuan [aut]
Maintainer: Xu Ren <xuren2120@gmail.com>
