#' @title An example of user input gene sets
#'
#' @description An example of user input gene sets
#' @docType data
#' @name GS.list
#' @usage GS.list
#' @format A list contains user input gene set
#' names and their corresponding genes
#' @keywords datasets
NULL

#' @title An example of user input cpg.pval
#'
#' @description An example of user input cpg.pval
#' @docType data
#' @name cpg.pval
#' @usage cpg.pval
#' @format A named vector contains p-values of each probe tested
#' @keywords datasets
NULL


#' @title An example of user user-supplied mapping between CpGs and genes
#'
#' @description An example of user user-supplied mapping between CpGs and genes
#' @docType data
#' @name CpG2Gene
#' @usage CpG2Gene
#' @format A data frame contains mapping between CpGs and genes
#' @keywords datasets
NULL




