## About

### Authors


[Xu Ren](https://github.com/reese3928) (xu.ren@stonybrook.edu)        
[Pei Fen Kuan](http://www.ams.stonybrook.edu/~pfkuan/) (peifen.kuan@stonybrook.edu)   

This Shiny app is an interface for Bioconductor package methylGSA. 

The Bioconductor package can be found [here](https://bioconductor.org/packages/release/bioc/html/methylGSA.html).    
The Bioconductor package vignette can be found [here](https://bioconductor.org/packages/release/bioc/vignettes/methylGSA/inst/doc/methylGSA-vignette.html).      
The `methylGSA` paper can be found [here](https://doi.org/10.1093/bioinformatics/bty892).


### Citation

Ren, X., & Kuan, P. F. (2019). methylGSA: a Bioconductor package and Shiny app for DNA methylation data length bias adjustment in gene set testing. Bioinformatics, 35(11), 1958-1959.

@article{ren2019methylgsa,    
title={methylGSA: a Bioconductor package and Shiny app for DNA methylation data length bias adjustment in gene set testing},    
author={Ren, Xu and Kuan, Pei Fen},    
journal={Bioinformatics},    
volume={35},    
number={11},    
pages={1958--1959},    
year={2019},    
publisher={Oxford University Press}    
}



